// include files needed for this module
#define INCL_PM		// PM library
#include <os2.h>

#include <dir.h>	// C library
#include <dos.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "myfuncs.h"	// a subset of my global routines library
#include "iconcal.h"	// this program
#include "wpcal.h"	// C++ translated version of Frank Wochatz WPCalendar.cmd

/* CONSTANTS */
// #define INFOWINDOW
static double dRelease = 1.00;

/* function prototypes */
int main ( void );
// performs the initation, object creation and termination activities
// returns 0 upon successful termination, 1 if pmassert exit option is selected

MRESULT EXPENTRY AboutDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
// displays product information
//	hwnd = client window handle
//	msg = message to be processed
//	mp1 = 1st message parameter
//	mp2 = 2nd message parameter
//	returns NULL if the I handled it, or the WinDefDlgProc result otherwise

void IconCalDlg( HWND hwnd );
// Creates the calander dialog where most of the wok takes place

MRESULT EXPENTRY IconCalDlgProc( HWND hwnd, ULONG  msg, MPARAM mp1, MPARAM mp2 );
// Handles the calander dialog messages

void Create( HWND hwnd );
// initializes the calander programs storing anchor blocks and
// sets up the presentation space

void GetDate( PMAIN_PARM pMain );
// gets the current month and year
//  pMain = calander program's gobal parameter block defined in calander.h

MRESULT EXPENTRY mainWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
// Main message handling routine for IconCal program

int main( void )
{
  BOOL bOK;		// Boolean used for return code
  HAB hab;		// PM anchor block handle
  HMQ hmq;		// Message queue handle
  QMSG qmsg;		// Message from message queue
  ULONG ulCtlData = 0;	// window create flags (none here for invisible window)
  HWND hwndFrame;	// Frame window handle
  HWND hwndClient;	// Client area window handle

hab = WinInitialize( 0 );             // normal PM application startup
hmq = WinCreateMsgQueue( hab, 0 );    // create a calander message queue

bOK = WinRegisterClass( hab,          // register calander client with system
			CLASSNAME,    // name to be registered
			mainWinProc,  // procedure to handle messages
			0,            // invisible window, so no class styles
			sizeof( PMAIN_PARM ) ); // reserve space for pointer
pmassert( hab, bOK ); // check if operation was successful.  If not, display
		      // a 'continue or 'exit' message box. The 'exit' option
		      // terminates the program returning 1
		      // all pmassert calls below perform this function

hwndFrame = WinCreateStdWindow( HWND_DESKTOP,     // create calander's window
				FS_SHELLPOSITION, // standard window
				&ulCtlData,       // no creation flags
				CLASSNAME,        // registered name
				CLASSNAME,        // title bar name
				0,                // no client window style
				(HMODULE)NULLHANDLE, // no resource
				ID_ICONCAL,      // frame window id
				&hwndClient );    // client window handle
  pmassert( hab, hwndFrame );
  pmassert( hab, hwndClient );

#ifdef INFOWINDOW		// handy real-time display for debugging
  CreateMessageWindow();	// see myfuncs.h for usage
#endif

  // process messages until calander handles a WM_QUIT message
  while( WinGetMsg( hab, &qmsg, (HWND)NULLHANDLE, 0, 0 ))
    WinDispatchMsg( hab, &qmsg );

#ifdef INFOWINDOW
  DestroyMessageWindow();	// remove debugging window
#endif

  WinDestroyWindow( hwndFrame );	// clean up
  WinDestroyMsgQueue( hmq );
  WinTerminate( hab );
  return 0;
}  /* main */

MRESULT EXPENTRY AboutDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  switch (msg)  {
  case WM_INITDLG:  // change the release no. to the current version
    if ( !SetStaticText( hwnd, (MPARAM) IDD_ABOUT_RELEASE, IDD_ABOUT_RELEASE,
			 20, "Release: %5.2f", dRelease ) )
      DisplayError( hwnd, "Set release", "AboutDlgProc" );
    return (MRESULT) TRUE;

  case WM_COMMAND:
    if ( SHORT1FROMMP( mp1 ) == MBID_OK )
    { if ( !WinDismissDlg( hwnd, TRUE ) )
	DisplayError( hwnd, "WinDismissDlg", "AboutDlgProc" );
      return (MRESULT) TRUE; } // mp1 if
  } // msg if

  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}  /* AboutDlgProc */

void IconCalDlg( HWND hwnd )
{
  HWND hwndDlg;
  PMAIN_PARM pMain;

  pMain = (PMAIN_PARM) WinQueryWindowULong( hwnd, QWL_USER );
  hwndDlg = WinLoadDlg( HWND_DESKTOP, hwnd, IconCalDlgProc, NULLHANDLE,
	     ID_ICONCALDLG, pMain );  // create the calander dialog
  if ( hwndDlg == NULLHANDLE )
    DisplayError( hwndDlg, "WinLoadDlg", "IconCalDlg" );
  if ( (BOOL) WinSendMsg( hwndDlg, WM_SETICON,
	      (MPARAM) WinLoadPointer( HWND_DESKTOP, 0, ICON_ICONCAL ), NULL )
	      == FALSE ) // set the system menu icon
    DisplayError( hwndDlg, "WM_SETICON", "IconCalDlg" );
  if ( !CenterDlg( hwndDlg, HWND_DESKTOP ) )
    DisplayError( hwndDlg, "CenterDlg", "IconCalDlg" );
  WinProcessDlg( hwndDlg );  // run the dialog untol 'Exit' is pressed
  if ( !WinDestroyWindow( hwndDlg ) )
    DisplayError( hwndDlg, "Destroy Window", "IconCalDlg" );
}  /* IconCalDlg */

MRESULT EXPENTRY IconCalDlgProc( HWND hwnd, ULONG  msg, MPARAM mp1, MPARAM mp2 )
{
  static PMAIN_PARM pMain = NULL;

  HWND hwndSysMenu;

  switch (msg)  {
  case WM_INITDLG:
    pMain = (PMAIN_PARM) mp2;
    pMain->hwndIconCal = hwnd;

    // add product information menu item to the system menu
    if ( ( hwndSysMenu =	// get system menu handle
		WinWindowFromID( hwnd, FID_SYSMENU ) ) == NULLHANDLE )
      DisplayError( hwnd, "FID_SYSMENU", "IconCalDlgProc" );
    else
    {
     MENUITEM menu;
      HWND hwndSubMenu;
      short rc;

      if ( !WinSendMsg( hwndSysMenu, MM_QUERYITEM,   // get system submenu hwnd
			MPFROM2SHORT( SC_SYSMENU, TRUE ), MPFROMP ( &menu ) ) )
	DisplayError( hwnd, "MM_QUERYITEM", "InventryDlgProc" );
      hwndSubMenu = menu.hwndSubMenu;

      menu.afAttribute = 0;	// setup to insert a separator
      menu.hItem = 0;
      menu.iPosition = MIT_END;
      menu.hwndSubMenu = NULLHANDLE;
      menu.afStyle = MIS_SEPARATOR;
      rc = (short) WinSendMsg( hwndSubMenu, MM_INSERTITEM,
			       MPFROMP( &menu ), 0 );
      if ( rc == MIT_ERROR || rc == MIT_MEMERROR )
	DisplayError( hwnd, "MM_INSERTITEM", "IconCalDlgProc" );

      menu.afStyle = MIS_TEXT | MIS_SYSCOMMAND; // insert the About item
      menu.id = IDM_ABOUT;
      rc = (short) WinSendMsg( hwndSubMenu, MM_INSERTITEM,
			       MPFROMP( &menu ),
			       MPFROMP( "Product Information" ) );
      if ( rc == MIT_ERROR || rc == MIT_MEMERROR )
	DisplayError( hwnd, "MM_INSERTITEM", "IconCalDlgProc" );
    } // hwndSysMenu else

    // initialize spin control & week start option values
    if ( !WinPostMsg( hwnd, MY_SET_VALUES, 0L, 0L ) )
      DisplayError( hwnd, "Post MY_SET_VALUES - inputs", "IconCalDlgProc" );

    if ( !WinSetFocus( HWND_DESKTOP, hwnd ) )
      DisplayError( hwnd, "WinSetFocus", "IconCalDlgProc" );
    break;	// WM_INITDLG

    // set initial values in date spin boxes & week start day
    case MY_SET_VALUES:
    if ( !SetNumberSpin( hwnd, mp1, IDD_ICONCAL_SPIN_DAY,
			 (long) pMain->ShowDate.da_day, 1, 31 ) )
	DisplayError( hwnd, "Set day spinner value", "IconCalDlgProc" );
    if ( !SetNumberSpin( hwnd, mp1, IDD_ICONCAL_SPIN_MONTH,
			 (long) pMain->ShowDate.da_mon, 1, 12 ) )
	DisplayError( hwnd, "Set month spinner value", "IconCalDlgProc" );
    if ( !SetNumberSpin( hwnd, mp1, IDD_ICONCAL_SPIN_YEAR,
			 (long) pMain->ShowDate.da_year, 1980, 2099 ) )
	DisplayError( hwnd, "Set year spinner value", "IconCalDlgProc" );
    WinCheckButton( hwnd, IDD_ICONCAL_WEEK_MON + pMain->lStart, TRUE );
    break;

  case WM_CLOSE:
    WinSendDlgItemMsg( hwnd, IDD_ICONCAL_DONE, BM_CLICK, 0, 0 );
    break;

  case WM_COMMAND:
    switch ( SHORT1FROMMP (mp1 ) ) { // Menu id, Push button id
    case IDD_ICONCAL_DONE:  // 'Exit' pressed, shut down dialog (and program)
      if ( !WinDestroyObject( pMain->hCMCicon ) )
	DisplayError( hwnd, "Destroy icon", "IconCalDlgProc" );
      if ( !WinPostMsg( pMain->hwndClient, WM_USER_EXIT, 0, 0 ) )
	DisplayError( hwnd, "Post WM_USER_EXIT", "IconCalDlgProc" );
      if ( !WinDismissDlg( hwnd, TRUE ) )
	DisplayError( hwnd, "WinDismissDlg - done", "IconCalDlgProc" );
      break;
    case IDD_ICONCAL_TODAY:	// set the icon to todays date
      GetDate( pMain );
      pMain->bToday = TRUE;	// this mode keeps the icon current
/*
#ifdef INFOWINDOW
  InfoMessage( "IconCalDlgProc", 0,
	"Today button: '%d/%d/%d'\r\n",
	pMain->ShowDate.da_day, pMain->ShowDate.da_mon,
	pMain->ShowDate.da_year );
#endif
*/
      // update the icon
      WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );
      break;
    case IDD_ICONCAL_NEXT_MONTH:
      pMain->ShowDate.da_mon++;
      if ( pMain->ShowDate.da_mon > 12 ) // wrap to next year
      { pMain->ShowDate.da_mon = 1;
	pMain->ShowDate.da_year++;
      } // pMain->ShowDate.da_mon if
      pMain->ShowDate.da_day =
	min( pMain->ShowDate.da_day, TotalDaysOfMonth( pMain->ShowDate.da_year,
	       pMain->ShowDate.da_mon ) );
      pMain->bToday = FALSE;
/*
#ifdef INFOWINDOW
  InfoMessage( "IconCalDlgProc", 0,
	"Next Month button: '%d/%d/%d'\r\n",
	pMain->ShowDate.da_day, pMain->ShowDate.da_mon,
	pMain->ShowDate.da_year );
#endif
*/
      WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );      break;
    case IDD_ICONCAL_NEXT_YEAR:
      pMain->ShowDate.da_year++;
      pMain->ShowDate.da_day =
	min( pMain->ShowDate.da_day, TotalDaysOfMonth( pMain->ShowDate.da_year,
	       pMain->ShowDate.da_mon ) );
      pMain->bToday = FALSE;
/*
#ifdef INFOWINDOW
  InfoMessage( "IconCalDlgProc", 0,
	"Next year button: '%d/%d/%d'\r\n",
	pMain->ShowDate.da_day, pMain->ShowDate.da_mon,
	pMain->ShowDate.da_year );
#endif
*/
      WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );      break;
    case IDD_ICONCAL_PREV_MONTH:
      pMain->ShowDate.da_mon--;
      if ( pMain->ShowDate.da_mon < 1 ) // go to previous year
      { pMain->ShowDate.da_mon = 12;
	pMain->ShowDate.da_year--;
      } // pMain->ShowDate.da_mon if
      pMain->ShowDate.da_day =
	min( pMain->ShowDate.da_day, TotalDaysOfMonth( pMain->ShowDate.da_year,
	       pMain->ShowDate.da_mon ) );
      pMain->bToday = FALSE;
/*
#ifdef INFOWINDOW
  InfoMessage( "IconCalDlgProc", 0,
	"Prev month button: '%d/%d/%d'\r\n",
	pMain->ShowDate.da_day, pMain->ShowDate.da_mon,
	pMain->ShowDate.da_year );
#endif
*/
      WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );      break;
    case IDD_ICONCAL_PREV_YEAR:
      pMain->ShowDate.da_year--;
      pMain->ShowDate.da_day =
	min( pMain->ShowDate.da_day, TotalDaysOfMonth( pMain->ShowDate.da_year,
	       pMain->ShowDate.da_mon ) );
      pMain->bToday = FALSE;
/*
#ifdef INFOWINDOW
  InfoMessage( "IconCalDlgProc", 0,
	"Prev year button: '%d/%d/%d'\r\n",
	pMain->ShowDate.da_day, pMain->ShowDate.da_mon,
	pMain->ShowDate.da_year );
#endif
*/
      WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );      break;
    case IDD_ICONCAL_UPDATE: // set item to a specific date
    {
      long lDay, lMonth, lYear;
      if ( !WinSendDlgItemMsg( hwnd, IDD_ICONCAL_SPIN_DAY,
			       SPBM_QUERYVALUE, MPFROMP( &lDay ), 0 ) )
	DisplayError( hwnd, "Get day spin value", "IconCalDlgProc" );
      if ( !WinSendDlgItemMsg( hwnd, IDD_ICONCAL_SPIN_MONTH,
			       SPBM_QUERYVALUE, MPFROMP( &lMonth ), 0 ) )
	DisplayError( hwnd, "Get month spin value", "IconCalDlgProc" );
      if ( !WinSendDlgItemMsg( hwnd, IDD_ICONCAL_SPIN_YEAR,
			       SPBM_QUERYVALUE, MPFROMP( &lYear ), 0 ) )
	DisplayError( hwnd, "Get year spin value", "IconCalDlgProc" );

      // make sure the month day actually exists
      pMain->ShowDate.da_day = min( lDay, TotalDaysOfMonth( lYear, lMonth ) );
      pMain->ShowDate.da_mon = lMonth;
      pMain->ShowDate.da_year = lYear;
/*
#ifdef INFOWINDOW
  InfoMessage( "IconCalDlgProc", 0,
	"Update icon button: '%d/%d/%d'\r\n",
	pMain->ShowDate.da_day, pMain->ShowDate.da_mon,
	pMain->ShowDate.da_year );
#endif
*/
      pMain->bToday = FALSE;
      WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );      }
      break;
      }  // MP1 SWITCH
    break;	// WM_COMMAND

  case WM_CONTROL:
    switch ( SHORT1FROMMP( mp1 ) )  {	// Control id
    case IDD_ICONCAL_WEEK_MON: // change to monday being the first week day
      pMain->lStart = 0;
      WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );
      break;
    case IDD_ICONCAL_WEEK_SUN: // change to sunday being the first week day
      pMain->lStart = 1;
      WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );
      break;
    } // mp1 switch
    break;	// WM_CONTROL

  case WM_SYSCOMMAND: // system menu
    switch ( SHORT1FROMMP (mp1 ) ) { // Menu id, Push button id
    case IDM_ABOUT:  // display product information
      WinDlgBox( HWND_DESKTOP, hwnd, AboutDlgProc, NULLHANDLE,
		 ID_ABOUTDLG, NULL );
      break;
    default:
      return WinDefDlgProc (hwnd, msg, mp1, mp2); }  // mp1 switch    break;

  default:
    return WinDefDlgProc (hwnd, msg, mp1, mp2);  } // msg switch

  return (MRESULT) TRUE;
}  /* IconCalDlgProc */

void Create( HWND hwnd )
{
  BOOL bOK;
  PMAIN_PARM pMain;

  {  // initialize window handles and main parameter block
    HAB hab;

    hab = WinQueryAnchorBlock( hwnd );  // get the inventry anchor block handle
    // allocate spave for the main inventry parameter block
    pMain = (PMAIN_PARM) calloc( 1, sizeof( MAIN_PARM ) );
    pmassert( hab, pMain );

    // save a pointer to the parameter block with the client window
    bOK = WinSetWindowULong( hwnd, QWL_USER, (ULONG) pMain );
    pmassert( hab, bOK  );

    // start setting the main parameter block values
    pMain->cb = sizeof( MAIN_PARM );
    pMain->hab = hab;
    pMain->hwndClient = hwnd;
    pMain->hwndFrame = WinQueryWindow( hwnd, QW_PARENT );
    // The inventory dialog handle will be set during initialization
    pMain->hwndIconCal = NULLHANDLE;
  }

  {  // create a presentation space for inventry to use with fond selection
    SIZEL sizel;

    // open a device context for the inventry client window
    pMain->hdcClient = WinOpenWindowDC( hwnd );
    pmassert( pMain->hab, pMain->hdcClient );
    // create presentation space with no extent
    sizel.cx = sizel.cy = 0;
    pMain->hpsClient = GpiCreatePS( pMain->hab, pMain->hdcClient, &sizel,
				    PU_LOMETRIC |	// units of 0.1 mm
				    GPIA_ASSOC );	// associate with hdc
    pmassert( pMain->hab, pMain->hpsClient );
  }

  // initialize parameters and display calander icon
  GetDate( pMain );
  pMain->PrevDate = pMain->ShowDate;
  pMain->lStart = 1;   // I prefer a sunday week start
  substrcpy( pMain->szLang, getenv( "LANG" ), 0, 2 );
  getcwd( pMain->szPath, MAXPATH );  // get the current working directory (cwd)

  // check that the icons directory is in cwd (not fatal, icon editor defaults)
  { char szIcons[STRLEN], szMessage[HUGESTR];
    sprintf( szIcons, "%s\\icons", pMain->szPath );
    if ( chdir( szIcons ) == -1 )  // try to change to the icons directory
    { sprintf( szMessage,
"Icons directory not found in current working directory: '%s'\
 The icon editor will be displayed instead of the day no. \
 To correct, please create per read.me", pMain->szPath );
       ErrorMessage( szMessage );  } // chdir if
    chdir( pMain->szPath ); // change back to the cwd
  }

  // create month calander icon, saving the handle
  pMain->hCMCicon = WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );
  // start timer that at LEN_ICONCAL_TIMER intervals triggers an icon update
  if ( !WinStartTimer( pMain->hab, hwnd, ID_ICONCAL_TIMER,
		       LEN_ICONCAL_TIMER ) )
    SysErrorMessage( pMain->hab, "WinStartTimer", "Create" );

  // load and start the calander dialog
  if ( !WinPostMsg( pMain->hwndClient, WM_USER_START, 0, 0 ) )
    SysErrorMessage( pMain->hab, "Post WM_USER_START", "Create" );
}  /* Create */

void GetDate( PMAIN_PARM pMain )
{
//  struct date today;
  struct tm *today;
  time_t seconds;

//  getdate( &today );  // crashes program for some reason (works in test prog!)
//  pMain->ShowDate = today;
  seconds = time( NULL );  // no. of seconds since 00:00:00 GMT 1-Jan-1970
  today = localtime( &seconds );  // translate into a structure that includes:
  pMain->ShowDate.da_day = today->tm_mday;		// todays day
  pMain->ShowDate.da_mon = today->tm_mon + 1;		// month
  pMain->ShowDate.da_year = today->tm_year + 1900;	// year
  pMain->bToday = TRUE;  // enable checking for date changes
  return;
}  /* GetDate */

MRESULT EXPENTRY mainWinProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  static PMAIN_PARM pMain;

  switch(msg) {

  // sent by WinCreateStdWindow in main() so that
  case WM_CREATE:
    Create( hwnd );	// window creation parameters are set
    pMain = (PMAIN_PARM) WinQueryWindowULong( hwnd, QWL_USER );
    break;

  // a timer event has occured
  case WM_TIMER:
    if ( (ULONG) mp1 == ID_ICONCAL_TIMER ) // check if it is my update timer
    { if ( pMain->bToday )	// only update if TRUE ('today' mode)
      { GetDate( pMain );	// get todays date
	if ( (pMain->PrevDate.da_day != pMain->ShowDate.da_day )
	  || (pMain->PrevDate.da_mon != pMain->ShowDate.da_mon )
	  || (pMain->PrevDate.da_year != pMain->ShowDate.da_year ) )
	{ pMain->PrevDate = pMain->ShowDate;  // date changed, update icon
	  WPCalendar( pMain->hab, pMain->ShowDate, pMain->lStart,
		  pMain->szPath, pMain->szLang );	} // pMain if
      } // bToday if
    } // ID_ICONCAL_TIMER
    break;

  // a message sent when the 'Exit' button is pressed the calander dialog
  case WM_USER_EXIT:	// all done, tell calander to close itself
    if ( !WinPostMsg( hwnd, WM_CLOSE, 0L, 0L ) )
      DisplayError( hwnd, "Post WM_CLOSE", "mainWinProc" );
    break;

  // sent by create when all initialization is done
  case WM_USER_START:
    // create and display the calander dialog, which takes control until
    // the dialog 'Exit' button is pressed or the window is closed
    IconCalDlg( hwnd );
    break;

  default:
    return WinDefWindowProc( hwnd, msg, mp1, mp2 );  }  /* Message switch */

  return (MRESULT) NULL;
}  /* mainWinProc */

