// procedures from my standard library

#define INCL_PM		// PM library
#define INCL_DOSPROCESS
#include <os2.h>

#include <dos.h>	// C library
#include <process.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "myfuncs.h"

// Used by the message window package
#define ERRORCLASSNAME  "ERRORWINDOW"
#define ID_ERRORWINDOW  101
#define WM_dfWIN_CLOSE ( WM_USER + 100 )  // internal messages for the error
#define WM_dfWIN_MSG   ( WM_dfWIN_CLOSE + 1 )  // window thread

typedef struct _MSG_WIN_PARM {
  HAB   hab;
  HWND  hwndClient;
  HWND  hwndFrame;
  HWND  hwndTitle;
  BOOL  bReady;
}
MSG_WIN_PARM;
typedef MSG_WIN_PARM *PMSG_WIN_PARM;

static PMSG_WIN_PARM pMsgWinParm = NULL;
static int iWinMsgThread;

// functions for starting & handling a message window
static void df_MsgWinThread( void  *pv );
MRESULT EXPENTRY df_MsgWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
LONG CreateOutlineFont(HPS hps, LONG lcid, PFATTRS pfattrs );
BOOL QueryFontAttr( HPS hps, char *szFacename, PFATTRS pfattrs );
BOOL ScaleOutlineFont( HPS hps, int iPointSize, int iPointWidth );

// procedure definitions

BOOL CenterDlg( HWND hwndDlg, HWND hwndObj )  // center Dlg on Obj
{
  long lHeight, lWidth, lXpos, lYpos;
  SWP swpDlg, swpObj;
						  // get dimensions & position
  if ( !WinQueryWindowPos( hwndDlg, (PSWP) &swpDlg ) ) return FALSE;
  if ( !WinQueryWindowPos( hwndObj, (PSWP) &swpObj ) ) return FALSE;
  lWidth = WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN );
  if ( lWidth == 0 ) return FALSE;
  lHeight = WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN );
  if ( lHeight == 0 ) return FALSE;

  lXpos = max( 0, swpObj.x + swpObj.cx / 2 - swpDlg.cx / 2 );
  lYpos = max( 0, swpObj.y + swpObj.cy / 2 - swpDlg.cy / 2 );
  if ( ( lXpos + swpDlg.cx ) > lWidth ) lXpos = max( 0, lWidth - swpDlg.cx );
  if ( ( lXpos + swpDlg.cy ) > lHeight ) lYpos = max( 0, lHeight - swpDlg.cy );

  return WinSetWindowPos( hwndDlg, HWND_TOP, lXpos, lYpos, 0, 0, SWP_MOVE );
}  /* CenterDlg */


BOOL CreateMessageWindow( void )
{
  if ( pMsgWinParm != NULL && pMsgWinParm->bReady )  // already running
    return TRUE;
  iWinMsgThread = _beginthread( df_MsgWinThread, 6*4096, NULL );
  if ( iWinMsgThread == 0 )  return FALSE;
  return TRUE;
}  /* CreateErrorMsgWin */

BOOL DestroyMessageWindow( void )
{
  if ( pMsgWinParm == NULL ) return FALSE; // not running
  WinSendMsg( pMsgWinParm->hwndClient, WM_dfWIN_CLOSE, 0, 0 );
  DosWaitThread( (TID *) &iWinMsgThread, DCWW_WAIT );
  return TRUE;
}  /* DestroyMessageWindow */

void DisplayError( HWND hwnd, char *szTitle, char *szSource )
{
  HAB hab;

  hab = WinQueryAnchorBlock( hwnd );
  if ( !SysErrorMessage( hab, szTitle, szSource ) )
    exit ( 1 );
} /* DisplayError */

short ErrorMessage( char *szMessage )
{
  return WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, szMessage,
	"Error", 0, MB_OKCANCEL | MB_ICONHAND );
} /* ErrorMessage */

long GetLastError( HAB hab, char *szMessage )
{
  ERRORID errid;
  PERRINFO perrinfo;

  perrinfo = WinGetErrorInfo( hab );
  if ( perrinfo == NULL ) return SEVERITY_NOERROR;
  errid = perrinfo->idError;

//??  sprintf( szMessage, "Sys Info:\n%s", perrinfo + perrinfo->offaoffszMsg );
  sprintf( szMessage, "Sys Info:\nError no %#x", ERRORIDERROR( errid ) );
  WinFreeErrorInfo( perrinfo );
  return ERRORIDSEV( errid );
}  /* GetLasterror */

short InfoMessage( char *szTitle, long fFlags, char *szFmt, ... )
{

  char *szMessage;                      // Buffer to build sting into
  FILE *Temp;                           // Temporary file to get size info
  long lLength, rc;                     // Length of Buffer & return code
  va_list  argptr;                      // Argument list pointer

  if ( ( Temp = tmpfile() ) == NULL )   // Create a temporary file
    lLength = 4096;                     // Problems so hope for the best
  else
  { va_start( argptr, szFmt );          // Initialize va_ functions
    lLength =
      vfprintf( Temp, szFmt, argptr );  // prints to file & get length
    va_end( argptr );                   // Close va_ functions
    if ( lLength == EOF )               // more problems
      lLength = 4095;
    lLength++;                          // leave room for '\0'
    rmtmp(); } /* Temp if */            // Remove temporary file

  szMessage = (char *) stralloc( lLength * sizeof( char ), "InfoMessage" );
  va_start( argptr, szFmt );            /* Initialize va_ functions     */
  vsprintf( szMessage, szFmt, argptr ); /* prints string to buffer      */
  va_end( argptr );                     /* Close va_ functions          */
					// display options
  if ( fFlags )                         // 1) mesaage box with user flags
    rc = WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, szMessage,
			  szTitle, 0, fFlags );

					// 2) message window, if open
  else if ( pMsgWinParm != NULL && pMsgWinParm->bReady )
  { WinSendMsg( pMsgWinParm->hwndClient, WM_dfWIN_MSG,
		MPFROMP ( szMessage ), MPFROMP ( szTitle ) );
    rc = MBID_OK; }

					// 3) message box where CANCEL terminate
  else if ( ( rc = WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, szMessage,
			   szTitle, 0, MB_OKCANCEL ) ) == MBID_CANCEL )
    exit( 1 );
  else
    rc = MBID_OK;
  myfree( szMessage );
  return rc;
} /* InfoMessage */

void *myfree( void *pMemory )
{
  if ( pMemory != NULL ) free ( pMemory );
  return NULL;
} /* myfree */

BOOL SetNumberSpin( HWND hwnd, MPARAM mp1, long lItemIDD, long lSelect,
		    long lLower, long lUpper )
{
  if ( ( SHORT1FROMMP( mp1 ) == lItemIDD )  ||  ( SHORT1FROMMP( mp1 ) == 0 ) )
  { if ( lLower >= lUpper ) return FALSE;
    if ( !WinSendDlgItemMsg( hwnd, lItemIDD, SPBM_SETLIMITS,
			     MPFROMLONG( lUpper ), MPFROMLONG( lLower ) ) )
      DisplayError( hwnd, "Set spinner limits", "SetNumberSpin" );
    lSelect = min( lUpper, max( lSelect, lLower ) );
    return (BOOL) WinSendDlgItemMsg( hwnd, lItemIDD, SPBM_SETCURRENTVALUE,
				     MPFROMLONG( lSelect ), 0 ); } // mp1 if IDD
  return FALSE;
}  /* SetNumberSpin */

BOOL SetStaticText( HWND hwnd, MPARAM mp1, long lItemIDD, long lSize,
		char *szFmt, ... )
{
  BOOL bOk = FALSE;
  char *szStr;                                  // Buffer to build sting into
  va_list  argptr;                              // Argument list pointer

  if ( ( SHORT1FROMMP( mp1 ) == lItemIDD )  ||  ( SHORT1FROMMP( mp1 ) == 0 ) )
  { szStr = (char *) stralloc( lSize * sizeof( char ) + 2, "SetStaticText" );
    va_start( argptr, szFmt );                  // Initialize va_ functions
    vsprintf( szStr, szFmt, argptr );           // prints string to buffer
    va_end( argptr );                           // Close va_ functions
    bOk = WinSetDlgItemText( hwnd, lItemIDD, szStr );
    free ( szStr );  } // mp1 if
  return bOk;
}  /* SetStaticText */

void *stralloc( long lAmount, char *szPlace )
{
  char *pPoint;

  pPoint = ( char *) calloc( 1, lAmount );
  if ( pPoint == NULL )
  { char szMessage[82];
    sprintf( szMessage, "Could not allocate %d bytes for %s", lAmount, szPlace);
    ErrorMessage( szMessage );
    return NULL; }  // pPoint if
  return (void *)pPoint;
}  /* stralloc */

char *substrcpy( char *dest, char *source, int start, int amt )
{
  int  i = 1, lth;
  char *dst;

  dst = dest;
  lth = strlen( source );
  if ( start < lth )
  { source += start;
    amt = min( ( lth - start ), amt );
    while ( (char *) ( ( *(dst++) ) = ( *(source++) ) ) &&
	    (char *) ( (i++) < amt ) );  } // start if
  *dst = '\0';
  return dest;
}  /* substrcpy */

BOOL SysErrorMessage( HAB hab, char *szTitle, char *szSource )
{
  BOOL bOk = TRUE;
  char szMessage[HUGESTR], szSevmess[STRLEN], szSysmess[BIGSTR];
  long severity;

  severity = GetLastError( hab, szSysmess );
  switch ( severity ) {
  case SEVERITY_UNRECOVERABLE:
    strcpy( szSevmess, "Unrecoverable - Terminating" );
    bOk = FALSE;
    break;
  case SEVERITY_SEVERE:
    strcpy( szSevmess, "Severe - CANCEL Terminates" );
    break;
  case SEVERITY_ERROR:
    strcpy( szSevmess, "Unknown severity - CANCEL Terminates" );
    break;
  case SEVERITY_WARNING:
    strcpy( szSevmess, "Warning - OK continues" );
    break;
  case SEVERITY_NOERROR:
  default:
    strcpy( szSevmess, "No error - OK continues" );
    break; } // severity switch

  sprintf( szMessage, "%s\nSource: '%s'\nMessage: '%s'\nSeverity: '%s'",
	   szTitle, szSource, szSysmess, szSevmess );
  return ( bOk && ( ErrorMessage( szMessage ) == MBID_OK ) );
} /* SysErrorMessage */

// Internal routines used by the message window
void df_MsgWinThread( void *pv )
{
  BOOL bOk;
  HAB hab;
  HMQ hmq;
  HWND hwndFrame;
  HWND hwndClient;
  QMSG qmsg;
  RECTL rectl;
  ULONG ulCtlData;

  hab = WinInitialize( 0 );
  if ( hab == NULLHANDLE )  ErrorMessage( "Can't open message window hab" );
  hmq = WinCreateMsgQueue( hab, 0 );
  pmassert( hab, hmq );

  bOk = WinRegisterClass( hab, ERRORCLASSNAME, df_MsgWinProc,
			  CS_SIZEREDRAW, sizeof( PMSG_WIN_PARM ) );
  pmassert( hab, bOk );

  ulCtlData = FCF_SIZEBORDER | FCF_TASKLIST | FCF_TITLEBAR;
  hwndFrame = WinCreateStdWindow( HWND_DESKTOP,
        WS_VISIBLE, &ulCtlData, ERRORCLASSNAME,
        NULL, 0, (HMODULE) NULLHANDLE, ID_ERRORWINDOW,
        &hwndClient );
  pmassert( hab, hwndFrame );
  pmassert( hab, hwndClient );

  WinQueryWindowRect( WinQueryDesktopWindow( hab, NULLHANDLE ), &rectl );
  bOk = WinSetWindowPos( hwndFrame, HWND_TOP,
			 rectl.xLeft, rectl.yTop - 75,
			 rectl.xRight - rectl.xLeft, 75,
                         SWP_SIZE | SWP_MOVE  );
  pmassert( hab, bOk );

  while( WinGetMsg ( hab, &qmsg, NULLHANDLE, 0, 0 ))
      WinDispatchMsg ( hab, &qmsg );

  bOk = WinDestroyWindow( hwndFrame );
  pmassert( hab, bOk );
  bOk = WinDestroyMsgQueue( hmq );
  pmassert( hab, bOk );
  bOk = WinTerminate( hab );
  pmassert( hab, bOk );
  _endthread();
}  /* df_MsgWinThread */

MRESULT EXPENTRY df_MsgWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  BOOL bOk;
  static char *szMessage = NULL;
  char *pEOL, *pPnt;
  FATTRS fattr;
  FONTMETRICS fm;
  HPS hps;
  int iPointSize;
  POINTL pointl;
  RECTL rectl;

  switch( msg ) {
  case WM_CREATE:
    if ( pMsgWinParm == NULL ) // should always be the case
      pMsgWinParm = (PMSG_WIN_PARM) stralloc( sizeof( MSG_WIN_PARM ),
                                              "df_MsgWinProc" );
    else
      ErrorMessage( "df_MsgWinProc\nWM_CREATE\npMsgWinParm != NULL" );
    pMsgWinParm->hab = WinQueryAnchorBlock( hwnd );
    pMsgWinParm->hwndClient = hwnd;
    pMsgWinParm->hwndFrame = WinQueryWindow( hwnd, QW_PARENT );
    pmassert( pMsgWinParm->hab, pMsgWinParm->hwndFrame );
    pMsgWinParm->hwndTitle =
      WinWindowFromID( pMsgWinParm->hwndFrame, FID_TITLEBAR );
    pmassert( pMsgWinParm->hab, pMsgWinParm->hwndTitle  );
    bOk = WinSetWindowText( pMsgWinParm->hwndTitle, "Message Window" );
    pmassert( pMsgWinParm->hab, bOk );
    pMsgWinParm->bReady = TRUE;
    if ( szMessage != NULL ) // also should always be the case
      ErrorMessage( "df_MsgWinProc\nWM_CREATE\nszMessage != NULL" );
    break;

  case WM_dfWIN_CLOSE:
    pMsgWinParm->bReady = FALSE;
    if ( szMessage != NULL )
    { free( szMessage );
      szMessage = NULL;; }
    bOk = WinPostMsg( hwnd, WM_QUIT, 0, 0 );
    pmassert( pMsgWinParm->hab, bOk );
    free ( pMsgWinParm );
    pMsgWinParm = NULL;
    break;

  case WM_dfWIN_MSG:
    pPnt = (char *) mp1;
    if ( szMessage != NULL ) free( szMessage );
    szMessage = (char *) stralloc( strlen( pPnt ) + 2, "df_MsgWinProc" );
    if ( szMessage != NULL )
    { strcpy( szMessage, pPnt );
      WinInvalidateRect ( hwnd, NULL, TRUE ); }
    if ( ( pPnt = (char *) mp2 ) != NULL )
      bOk = WinSetWindowText( pMsgWinParm->hwndTitle, pPnt );
    else
      bOk = WinSetWindowText( pMsgWinParm->hwndTitle, "Message Window" );
    pmassert( pMsgWinParm->hab, bOk );
    break;

  case WM_PAINT:
    bOk = WinQueryWindowRect( hwnd, &rectl );
    pmassert( pMsgWinParm->hab, bOk );
    hps = WinBeginPaint( hwnd, 0, 0 );
    pmassert( pMsgWinParm->hab, hps );
    bOk = WinFillRect( hps, &rectl, CLR_BACKGROUND );
    pmassert( pMsgWinParm->hab, bOk );
    if ( szMessage != NULL )
    { CreateOutlineFont( hps, 1, &fattr );
      GpiSetCharSet( hps, 1 );
      iPointSize =  (int) ( 10 * 10.0 );
      ScaleOutlineFont( hps, iPointSize, iPointSize );
      GpiQueryFontMetrics ( hps, sizeof( FONTMETRICS ), &fm );
      pointl.x = 1;
      pointl.y = rectl.yTop - fm.lMaxBaselineExt;
      pPnt = szMessage; // point to first or only line
      while ( ( pEOL = strchr( pPnt, '\n' ) ) != NULL )// multiline message
      { *pEOL = '\0';   // replace CR with NULL and draw substring
        bOk = GpiCharStringAt( hps, &pointl, strlen( pPnt ), pPnt );
        pmassert( pMsgWinParm->hab, bOk );
	pointl.y -= fm.lMaxBaselineExt; // move down one line
        *pEOL = '\n';   // replace CR
        pEOL++;         // move beyond
        pPnt = pEOL; } /* while */    // point to next line
      bOk = GpiCharStringAt( hps, &pointl, strlen( pPnt ), pPnt );
      pmassert( pMsgWinParm->hab, bOk );
    } /* szMessage if */
    bOk = WinEndPaint( hps );
    pmassert( pMsgWinParm->hab, bOk );
    break;

  default:
    return WinDefWindowProc( hwnd, msg, mp1, mp2 );  }  /* msg switch */

  return (MRESULT) NULL;
}  /* df_MsgWinProc */

LONG CreateOutlineFont(HPS hps, LONG lcid, PFATTRS pfattrs )
{
  LONG lReturn;

  if ( pfattrs->usRecordLength == 0 )   // set up a default
  { memset( pfattrs, 0, sizeof( FATTRS ) );
    pfattrs->usRecordLength = sizeof (FATTRS);
    pfattrs->fsFontUse = FATTR_FONTUSE_OUTLINE | FATTR_FONTUSE_TRANSFORMABLE;
    strcpy ( pfattrs->szFacename, "Times New Roman" );
    if ( !QueryFontAttr( hps, "Times New Roman", pfattrs ) )
      return GPI_ERROR; } /* PFATTR IF */

  lReturn = GpiCreateLogFont( hps, NULL, lcid, pfattrs );

  if ( ( lReturn == FONT_DEFAULT ) && ( pfattrs->usCodePage == 0 ) )
  { pfattrs->usCodePage = 65400;        // If no match, try a symbol code page
    lReturn = GpiCreateLogFont ( hps, NULL, lcid, pfattrs ) ;  }
  return lReturn ;
}  /* CreateOutlineFont */

BOOL QueryFontAttr( HPS hps, char *szFacename, PFATTRS pfattrs )
{
  long lCount = 0, lFonts;
  PFONTMETRICS pfmMetrics;

  lFonts = GpiQueryFonts( hps, QF_PUBLIC, szFacename, &lCount,
                      sizeof( FONTMETRICS ), NULL );
  if ( lFonts == GPI_ALTERROR )
  { ErrorMessage( "QueryFontAttr - GpiQueryFonts error" );
    return FALSE; }
  if ( lFonts == 0 )
  { ErrorMessage( "No Fonts found" );
    return FALSE; }

// InfoMessage( "QueryFontAttr", 0, "Fonts found: %d", lFonts );

  if ( ( pfmMetrics = (PFONTMETRICS) stralloc( lFonts * sizeof( FONTMETRICS ),
                                               "QueryFontAttr" ) ) == NULL )
    return FALSE;

  lCount = lFonts;
  lFonts = GpiQueryFonts( hps, QF_PUBLIC, szFacename, &lCount,
                      sizeof( FONTMETRICS ),  pfmMetrics );
  if ( lFonts == GPI_ALTERROR )
  { ErrorMessage( "QueryFontAttr - GpiQueryFonts error" );
    free( pfmMetrics );
    return FALSE; }
/*
InfoMessage( "QueryFontAttr", 0, "Facename: '%s'\nMatch: %d\nRegistry: %d\n\
Code page: %d", pfmMetrics->szFacename, pfmMetrics->lMatch,
pfmMetrics->idRegistry, pfmMetrics->usCodePage );
*/
  pfattrs->usRecordLength = sizeof( FATTRS );
  strcpy( pfattrs->szFacename, pfmMetrics->szFacename );
  pfattrs->lMatch = pfmMetrics->lMatch;
  pfattrs->idRegistry = pfmMetrics->idRegistry;
  pfattrs->usCodePage = pfmMetrics->usCodePage;
  pfattrs->lMaxBaselineExt = pfmMetrics->lMaxBaselineExt;
  pfattrs->lAveCharWidth = pfmMetrics->lAveCharWidth;

  free( pfmMetrics );
  return TRUE;
}  /* QueryFontAttr */

BOOL ScaleOutlineFont( HPS hps, int iPointSize, int iPointWidth )
{
  HDC hdc ;
  long xRes, yRes ;
  POINTL aptl[2] ;
  SIZEF  sizef ;

  hdc = GpiQueryDevice( hps );  // Get font resolution in pixels per inch
  if ( hdc == HDC_ERROR || hdc == NULLHANDLE ) return FALSE;
  if ( !DevQueryCaps( hdc, CAPS_HORIZONTAL_FONT_RES, 1, &xRes ) ) return FALSE;
  if ( !DevQueryCaps( hdc, CAPS_VERTICAL_FONT_RES,   1, &yRes ) ) return FALSE;

  if ( iPointWidth == 0 )       // Find desired font size in pixels
    iPointWidth = iPointSize;

  aptl[0].x = 0;
  aptl[0].y = 0;
  aptl[1].x = ( 16 * xRes * iPointWidth + 360 ) / 720;
  aptl[1].y = ( 16 * yRes * iPointSize  + 360 ) / 720;
                                 // Convert to page coordinates
  if ( !GpiConvert (hps, CVTC_DEVICE, CVTC_PAGE, 2L, aptl) ) return FALSE;

  sizef.cx = ( aptl[1].x - aptl[0].x ) << 12;   // Set the character box
  sizef.cy = ( aptl[1].y - aptl[0].y ) << 12;
  return GpiSetCharBox ( hps, &sizef );
}  /* ScaleOutlineFont */

